import csv
import json
import hashlib
from .adapter import Adapter as BaseAdapter
from ..constants import (ALLOWED_HTML_TAGS, HTML_FIELDS,
                         WOO_IMP_EXP_PRODUCT_FIELDS)

DP_TRACK_URL_BASE = 'https://ant.sr/dpsounds-audio'
DP_TRACK_CATEGORIES = json.dumps([{'name': 'Music', 'slug': 'music'}])
DP_TRACK_ATTRIBUTES = ['Time', 'Origin']
DP_TRACK_PRICE = 1


class DpsoundsAdapter(BaseAdapter):
    """Input handler for spreadsheets received from my dad
    for dpsounds.com.
    Headers:
     "Source Part","Source Track","Track",
     "Title","Time","Origin","Tags",
     "Alternative Takes","Compilation".
    """

    def parse(self, filename):
        """Parse a CSV file for DPSounds.com."""
        def parse_row(r):
            """Parse a row of Dad's CSV file and rearrange the fields for
            WooCommerce."""
            name = "Track {}".format(r['Track'])
            if r['Title']:
                name += " - {}".format(r['Title'])
            tags = ','.join([t.strip() for t in r['Tags'].split(',')])
            file_url = "{url_base}/{part}-{track:02d}.mp3".format(
                url_base=DP_TRACK_URL_BASE,
                part=r['Source Part'],
                track=int(r['Source Track']))
            filehash = hashlib.md5(file_url.encode('utf-8')).hexdigest()
            files = json.dumps({
                filehash: {'name': name, 'file': file_url}})
            # TODO: alternative takes
            # TODO: compilation (album)
            # TODO: Description text to include attributes.
            attributes = [{
                    'name': attr,
                    'value': r[attr],
                    'position': idx,
                    'is_visible': '1',
                    'is_variation': '0',
                    'is_taxonomy': '0'
                } for idx, attr in enumerate(DP_TRACK_ATTRIBUTES)]
            product = dict(WOO_IMP_EXP_PRODUCT_FIELDS)
            product.update({
                "Product Name": name,
                "Tags": tags,
                "Categories": DP_TRACK_CATEGORIES,
                "Downloadable": 'yes',
                "Virtual": 'yes',
                "Sold Individually": 'yes',
                "Downloadable Files": files,
                "Download Type": "music",
                "Price": DP_TRACK_PRICE,
                "Regular Price": DP_TRACK_PRICE,
                "Attributes": json.dumps(attributes)
            })
            return product

        with open(filename) as csv_in:
            csv_reader = csv.DictReader(csv_in)
            raw_category_names = []
            for row in csv_reader:
                product = parse_row(row)
                self.products.append(product)

adapter_cls = DpsoundsAdapter
